'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _identity = require('ramda/src/identity');

var _identity2 = _interopRequireDefault(_identity);

var _mergeWithKey = require('ramda/src/mergeWithKey');

var _mergeWithKey2 = _interopRequireDefault(_mergeWithKey);

var _prop = require('ramda/src/prop');

var _prop2 = _interopRequireDefault(_prop);

var _filter = require('ramda/src/filter');

var _filter2 = _interopRequireDefault(_filter);

var _sequence = require('ramda/src/sequence');

var _sequence2 = _interopRequireDefault(_sequence);

var _isEmpty = require('ramda/src/isEmpty');

var _isEmpty2 = _interopRequireDefault(_isEmpty);

var _ifElse = require('ramda/src/ifElse');

var _ifElse2 = _interopRequireDefault(_ifElse);

var _chain = require('ramda/src/chain');

var _chain2 = _interopRequireDefault(_chain);

var _compose = require('ramda/src/compose');

var _compose2 = _interopRequireDefault(_compose);

var _pick = require('ramda/src/pick');

var _pick2 = _interopRequireDefault(_pick);

var _merge = require('ramda/src/merge');

var _merge2 = _interopRequireDefault(_merge);

var _map = require('ramda/src/map');

var _map2 = _interopRequireDefault(_map);

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _data = require('data.either');

var _data2 = _interopRequireDefault(_data);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Right = _data2.default.Right,
    Left = _data2.default.Left;

/**
 *
 * curried function for creating a new object containing an empty array for every key
 * createDefaultValidation({'name': '1', 'random': '2'})
 * outputs: {name: [], random: []}
 */

var createDefaultValidation = (0, _map2.default)(function () {
  return [];
}

/**
 *
 * merge default validation rules with actual validation rules
 * @param input
 * @param validations
 * @example
 *    createBaseValidation(
 *      {name: [], random: []},
 *      {name: [[x => x > 1, 'some msg']]}
 *    ) // {name: [[x => x > 1, 'some msg']], random: []})
 *
 *    createBaseValidation(
 *      {name: []},
 *      {name: [[x => x > 1, 'some msg']], random: [[x => x + 1, 'random msg']]}
 *    ) // {name: [[x => x > 1, 'some msg']]}
 */
);var createBaseValidation = function createBaseValidation(input, validations) {
  return (0, _merge2.default)(createDefaultValidation(input), (0, _pick2.default)(Object.keys(input), validations)

  /**
   * higher order function expecting a predicate function and error message tuple.
   * Returns a new function expecting the value and the input object.
   * the predicate function is applied with the input and result object.
   * if predicate function is truthy an Either.Right is rerturned else an Else.Left
   *
   * @param {Function} predFn the predicate function to be run
   * @param {string} e the error message
   */
  );
};var makePredicate = function makePredicate(_ref) {
  var _ref2 = _slicedToArray(_ref, 2),
      predFn = _ref2[0],
      e = _ref2[1];

  return function (a, inputs) {
    return predFn(a, inputs) ? Left(a) : Right(e

    /**
     * applies all predicates with the input value and inputs objects
     *
     * @param {*} input the value to be tested
     * @param {Array} validations all validations to be applied with the input
     * @param {Object} all object containing all input values
     */
    );
  };
};var runPredicates = function runPredicates(_ref3) {
  var _ref4 = _slicedToArray(_ref3, 3),
      input = _ref4[0],
      validations = _ref4[1],
      all = _ref4[2];

  return (0, _map2.default)(function (predFn) {
    return predFn(input, all);
  }, (0, _map2.default)(makePredicate, validations)

  /**
   * validator expects a transformation function to run on the results of the predicates and the sequence.
   * Depending on what is needed, one can transform the Either result.
   *
   * @param {Function} transform to apply the transformation
   */
  );
};var validator = function validator(transform) {
  return (0, _map2.default)((0, _compose2.default)(transform, (0, _chain2.default)((0, _ifElse2.default)(_isEmpty2.default, Right, Left)), (0, _sequence2.default)(_data2.default.of), (0, _filter2.default)((0, _prop2.default)('isRight')), runPredicates // eslint-disable-line comma-dangle
  )

  /**
   *
   * @param {Object} input the provided input object
   * @param {Object} validations the validations for the input keys
   */
  );
};var makeValidationObject = function makeValidationObject(input, validations) {
  return (0, _mergeWithKey2.default)(function (k, l, r) {
    return [l, r, input];
  }, input, createBaseValidation(input, validations)

  /**
   *
   * @param transform
   */
  );
};var createValidation = function createValidation() {
  var transform = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _identity2.default;
  return (0, _compose2.default)(validator(transform), makeValidationObject);
};

exports.default = createValidation;